@@ -36,6 +36,8 @@ To install GEM on SGI:\n\n\
 1) untar the GEM package so that it is located at pd/gem\n\n\
 2) run GEM_INSTALL.sh\n    - this will put everything where it should be\n\n\n\
 3) Read the FAQ for instructions to run Pd and GEM\n\n\
+4) Check the release notes for changes and usage information\n\n\
+5) Read the manual html files to learn how to use GEM\n\n\
 mark@danks.org" > GEM.README
 
 # set the permissions on the install file
@@ -55,7 +57,7 @@ compress gem-$1.tar
 ################
 echo Tarring binary version up...
 
-tar cvf gem-$1bin.tar gem/Gem.pd_irix5 gem/GEM.LICENSE.TERMS gem/GnuGPL.LICENSE gem/GEM.README gem/GEM_INSTALL.sh gem/doc/* gem/help/* gem/examples/* 
+tar cvf gem-$1bin.tar gem/Gem.pd_irix5 gem/GEM.LICENSE.TERMS gem/GnuGPL.LICENSE gem/GEM.README gem/GEM_INSTALL.sh gem/doc/* gem/help/* gem/examples/* gem/manual/*
 
 compress gem-$1bin.tar
 
@@ -63,6 +65,6 @@ compress gem-$1bin.tar
 #############
 echo Tarring DSO version up...
 
-tar cvf gem-$1dso.tar gem/Gem.pd_irix5 gem/GEM.LICENSE.TERMS gem/GnuGPL.LICENSE gem/GEM.README gem/GEM_INSTALL.sh gem/doc/* 
+tar cvf gem-$1dso.tar gem/Gem.pd_irix5 gem/GEM.LICENSE.TERMS gem/GnuGPL.LICENSE gem/GEM.README gem/GEM_INSTALL.sh gem/doc/* gem/manual/*
 
 compress gem-$1dso.tar
