@@ -4,7 +4,7 @@ LOG
 
     The state to pass among GEM objects
 
-    Copyright (c) 1997-1998 Mark Danks. mark@danks.org
+    Copyright (c) 1997-1999 Mark Danks. mark@danks.org
     For information on usage and redistribution, and for a DISCLAIMER OF ALL
     WARRANTIES, see the file, "GEM.LICENSE.TERMS" in this distribution.
 
@@ -13,7 +13,7 @@ LOG
 #ifndef INCLUDE_GEMCACHE_H_
 #define INCLUDE_GEMCACHE_H_
 
-#include "Base/CPPExtern.h"
+#include "Base/GemExportDef.h"
 
 class gemhead;
 
