@@ -6,7 +6,7 @@
 //
 // Implementation file
 //
-//    Copyright (c) 1997-1998 Mark Danks.
+//    Copyright (c) 1997-1999 Mark Danks.
 //    For information on usage and redistribution, and for a DISCLAIMER OF ALL
 //    WARRANTIES, see the file, "GEM.LICENSE.TERMS" in this distribution.
 //
@@ -17,24 +17,26 @@
 #ifdef unix
 #include <sys/time.h>
 #include <GL/glx.h>
+#include <X11/Xlib.h>
 #elif _WINDOWS
 #include <stdlib.h>
+// I hate Microsoft...I shouldn't have to do this!
 #endif
 
-#include <GL/glu.h>
-
 #include "GemVersion.h"
 #include "GemState.h"
-#include "Controls/gemhead.h"
 #include "GemWinCreate.h"
 #include "GemEvent.h"
 
+#include "Controls/gemhead.h"
+
 static WindowInfo gfxInfo;
 static WindowInfo constInfo;
 
 // static member data
 int GemMan::m_height = 500;
 int GemMan::m_width = 500;
+int GemMan::m_border = 1;
 int GemMan::m_buffer = 2;
 int GemMan::m_profile = 0;
 GLfloat GemMan::m_clear_color[4];
@@ -43,6 +45,7 @@ GLfloat GemMan::m_mat_specular[4];
 GLfloat GemMan::m_mat_shininess;
 int GemMan::m_windowState = 0;
 int GemMan::m_windowNumber = 0;
+float GemMan::m_perspect[6];
 
 // static data
 static const int NUM_LIGHTS = 8;   	// the maximum number of lights
@@ -77,9 +80,15 @@ class gemheadLink
 static int createConstWindow();
 // static int destroyConstWindow();
 
-#ifdef _WINDOWS
+GEM_EXTERN void gemAbortRendering()
+{
+	GemMan::stopRendering();
+}
+
 static t_clock *s_windowClock = NULL;
 static int s_windowDelTime = 10;
+
+#ifdef _WINDOWS
 static int s_windowRun = 0;
 static int s_singleContext = 0;
 
@@ -89,7 +98,8 @@ static int s_singleContext = 0;
 /////////////////////////////////////////////////////////
 static void dispatchGemWindowMessages()
 {
-    if (!s_windowRun) return;
+    if (!s_windowRun)
+		return;
 
     MSG msg;
     while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE) == TRUE)
@@ -99,19 +109,51 @@ static void dispatchGemWindowMessages()
     }
     clock_delay(s_windowClock, s_windowDelTime);
 }
-#endif
+#elif unix 
+static void dispatchGemWindowMessages()
+{
+    WindowInfo win; 
+    XEvent event; 
+    XButtonEvent* eb = (XButtonEvent*)&event; 
+    win = GemMan::getWindowInfo(); 
+
+    while 
+		(XCheckWindowEvent(win.dpy,win.win,PointerMotionMask | ButtonMotionMask |
+                             ButtonPressMask | ButtonReleaseMask,&event))
+	{
+        switch (event.type)
+		{
+			case ButtonPress: 
+				triggerButtonEvent(eb->button-1, 1, eb->x, eb->y); 
+				break; 
+			case ButtonRelease: 
+				triggerButtonEvent(eb->button-1, 0, eb->x, eb->y); 
+				break; 
+			case MotionNotify: 
+				triggerMotionEvent(eb->x, eb->y); 
+				break; 
+			default:
+				break; 
+		}
+    }
+    clock_delay(s_windowClock, s_windowDelTime);  
+} 
+#endif // for Unix
 
 static void resizeCallback(int xSize, int ySize, void *)
 {
     float xDivy = (float)xSize / (float)ySize;
 
     // setup the viewpoint
+    glViewport(0, 0, xSize, ySize);
+	// setup the matrices
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
-    glFrustum(-xDivy, xDivy, -1.0, 1.0, 1.0, 20.0);
-    gluLookAt(0.0, 0.0, 4.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
+    glFrustum(GemMan::m_perspect[0] * xDivy, GemMan::m_perspect[1] * xDivy,	// left, right
+				GemMan::m_perspect[2], GemMan::m_perspect[3],				// bottom, top
+				GemMan::m_perspect[4], GemMan::m_perspect[5]);				// front, back
+ 
     glMatrixMode(GL_MODELVIEW);
-    glViewport(0, 0, xSize, ySize);
 }
 
 /////////////////////////////////////////////////////////
@@ -125,19 +167,21 @@ static void resizeCallback(int xSize, int ySize, void *)
 void GemMan :: initGem()
 {
     static int alreadyInit = 0;
-    if (alreadyInit) return;
+    if (alreadyInit)
+		return;
     alreadyInit = 1;
     
     // clear the light array
-    for (int i = 0; i < NUM_LIGHTS; i++) s_lights[i] = 0;
+    for (int i = 0; i < NUM_LIGHTS; i++)
+		s_lights[i] = 0;
     
     m_clear_color[0] = 0.0;
     m_clear_color[1] = 0.0;
     m_clear_color[2] = 0.0;
     m_clear_color[3] = 0.0;
-    m_mat_ambient[0] = 0.1f;
-    m_mat_ambient[1] = 0.1f;
-    m_mat_ambient[2] = 0.1f;
+    m_mat_ambient[0] = 0.0f;
+    m_mat_ambient[1] = 0.0f;
+    m_mat_ambient[2] = 0.0f;
     m_mat_ambient[3] = 1.0f;
     m_mat_specular[0] = 1.0;
     m_mat_specular[1] = 1.0;
@@ -151,6 +195,14 @@ void GemMan :: initGem()
     post("GEM: ver: %s", GEM_VERSION);
     post("GEM: compiled: " __DATE__);
 
+	// setup the perspective values
+	m_perspect[0] = -1.f;	// left
+	m_perspect[1] =  1.f;	// right
+	m_perspect[2] = -1.f;	// bottom
+	m_perspect[3] =  1.f;	// top
+	m_perspect[4] =  1.f;	// front
+	m_perspect[5] = 20.f;	// back
+
 #ifdef unix
     Display *dummyDpy;
     if ( (dummyDpy = XOpenDisplay(NULL)) == NULL)
@@ -167,7 +219,6 @@ void GemMan :: initGem()
 	    return;
     }
 #elif _WINDOWS
-    s_windowClock = clock_new(NULL, (t_method)dispatchGemWindowMessages);
 	// can we only have one context?
 	if (getenv("GEM_SINGLE_CONTEXT") &&
 		!strcmp("1", getenv("GEM_SINGLE_CONTEXT")))
@@ -179,6 +230,8 @@ void GemMan :: initGem()
 	}
 #endif
 
+    s_windowClock = clock_new(NULL, (t_method)dispatchGemWindowMessages);
+
     if (!createConstWindow())
     {
     	error("GEM: A serious error occured creating const Context");
@@ -273,6 +326,17 @@ void GemMan :: resetValues()
 	    glDisable(GL_NORMALIZE);
 	    glShadeModel(GL_FLAT);
     }
+
+	// setup the transformation matrices
+    float xDivy = (float)m_width / (float)m_height;
+    glMatrixMode(GL_PROJECTION);
+    glLoadIdentity();
+    glFrustum(m_perspect[0] * xDivy, m_perspect[1] * xDivy,	// left, right
+				m_perspect[2], m_perspect[3],				// bottom, top
+				m_perspect[4], m_perspect[5]);				// front, back
+    
+	glMatrixMode(GL_MODELVIEW);
+    gluLookAt(0.0, 0.0, 4.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
 }
 
 /////////////////////////////////////////////////////////
@@ -313,6 +377,14 @@ void GemMan :: resetState()
     m_height = 500;
     m_width = 500;
     m_buffer = 2;
+
+	// setup the perspective values
+	m_perspect[0] = -1.f;	// left
+	m_perspect[1] =  1.f;	// right
+	m_perspect[2] = -1.f;	// bottom
+	m_perspect[3] =  1.f;	// top
+	m_perspect[4] =  1.f;	// front
+	m_perspect[5] = 20.f;	// back
 }
 
 /////////////////////////////////////////////////////////
@@ -326,15 +398,18 @@ void GemMan :: render(void *)
 	static float countFreq = 0;
 #endif
 
-    if (!m_windowState) return;
+    if (!m_windowState)
+		return;
 
 	// are we profiling?
 #ifdef _WINDOWS
 	if (firstTime)
 	{
 		LARGE_INTEGER freq;
-		if (!QueryPerformanceFrequency(&freq)) countFreq = 0;
-		else countFreq = (float)(freq.QuadPart);
+		if (!QueryPerformanceFrequency(&freq))
+			countFreq = 0;
+		else
+			countFreq = (float)(freq.QuadPart);
 		firstTime = 0;
 	}
 	LARGE_INTEGER startTime;
@@ -364,7 +439,8 @@ void GemMan :: render(void *)
     }
 
     // only want to swap if we are in double buffer mode
-    if (GemMan::m_buffer == 2) swapBuffers();
+    if (GemMan::m_buffer == 2)
+		swapBuffers();
 
 	// are we profiling?
 	if (m_profile == 1 || m_profile == 2)
@@ -392,7 +468,8 @@ void GemMan :: render(void *)
 
     // only keep going if no one set the s_hit (could be hit if scheduler gets
     //	    ahold of a stopRendering command)
-    if (!s_hit) clock_delay(s_clock, s_deltime);
+    if (!s_hit)
+		clock_delay(s_clock, s_deltime);
 }
 
 /////////////////////////////////////////////////////////
@@ -407,7 +484,8 @@ void GemMan :: startRendering()
     	return;
     }
     
-    if (s_rendering) return;
+    if (s_rendering)
+		return;
     
     post("GEM: Start rendering");
     
@@ -422,7 +500,8 @@ void GemMan :: startRendering()
     s_rendering = 1;
     
     // if only single buffering then just return
-    if (GemMan::m_buffer == 1) return;
+    if (GemMan::m_buffer == 1)
+		return;
 
     render(NULL);
 }
@@ -465,15 +544,6 @@ void GemMan :: windowInit()
     glClearColor(m_clear_color[0], m_clear_color[1], m_clear_color[2], m_clear_color[3]);
  
     resetValues();
-
-    // setup the viewpoint
-    float xDivy = (float)m_width / (float)m_height;
-    glMatrixMode(GL_PROJECTION);
-    glLoadIdentity();
-    glFrustum(-xDivy, xDivy, -1.0, 1.0, 1.0, 20.0);
-    gluLookAt(0.0, 0.0, 4.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
-    glMatrixMode(GL_MODELVIEW);
-    glViewport(0, 0, m_width, m_height);
 }
 
 /////////////////////////////////////////////////////////
@@ -492,6 +562,7 @@ int GemMan :: createWindow()
     if ( m_windowState ) return(0);
 
     WindowHints myHints;
+	myHints.border = m_border;
     myHints.buffer = m_buffer;
     myHints.width = m_width;
     myHints.height = m_height;
@@ -508,8 +579,9 @@ int GemMan :: createWindow()
 
     windowInit();
     
-#ifdef _WINDOWS
     clock_delay(s_windowClock, s_windowDelTime);
+
+#ifdef _WINDOWS
     s_windowRun = 1;
 #endif
     return(1);
@@ -530,7 +602,8 @@ void GemMan :: destroyWindow()
     if (!m_windowState) return;
 
     stopRendering();
-    
+    clock_unset(s_windowClock);
+
     glFlush();
     glFinish();
 
@@ -566,6 +639,7 @@ int createConstWindow()
 #endif
 
     WindowHints myHints;
+	myHints.border = 1;
     myHints.buffer = 1;
     myHints.width = GemMan::m_width;
     myHints.height = GemMan::m_height;
@@ -617,7 +691,19 @@ void GemMan :: swapBuffers()
     glLoadIdentity();
 
     if (GemMan::m_buffer == 1)
+	{
         glFlush();
+		// setup the transformation matrices
+		float xDivy = (float)m_width / (float)m_height;
+		glMatrixMode(GL_PROJECTION);
+		glLoadIdentity();
+		glFrustum(m_perspect[0] * xDivy, m_perspect[1] * xDivy,	// left, right
+				  m_perspect[2], m_perspect[3],					// bottom, top
+				  m_perspect[4], m_perspect[5]);				// front, back
+    
+		glMatrixMode(GL_MODELVIEW);
+		gluLookAt(0.0, 0.0, 4.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
+	}
 }
 
 /////////////////////////////////////////////////////////
@@ -648,51 +734,57 @@ void GemMan :: frameRate(float framespersecond)
 // requestLight
 //
 /////////////////////////////////////////////////////////
-GLenum GemMan :: requestLight()
+GLenum GemMan :: requestLight(int specific)
 {
     int i = 0;
-    while(s_lights[i])
-    {
-	    i++;
-	    if (i >= NUM_LIGHTS)
-	    {
-	        error("GEM: Unable to allocate light");
-	        return((GLenum)0);
-	    }
-    }
-    s_lights[i] = 1;
+	if (specific > 0)
+		i = specific - 1;
+	else
+	{
+		while(s_lights[i])
+		{
+			i++;
+			if (i >= NUM_LIGHTS)
+			{
+				error("GEM: Unable to allocate light");
+				return((GLenum)0);
+			}
+		}
+	}
+    s_lights[i]++;
+	GLenum retLight;
     switch(i)
     {
 	    case (0) :
-	        return(GL_LIGHT0);
-	        // break;
+	        retLight = GL_LIGHT0;
+	        break;
 	    case (1) :
-	        return(GL_LIGHT1);
-	        // break;
+	        retLight = GL_LIGHT1;
+	        break;
 	    case (2) :
-	        return(GL_LIGHT2);
-	        // break;
+	        retLight = GL_LIGHT2;
+	        break;
 	    case (3) :
-	        return(GL_LIGHT3);
-	        // break;
+	        retLight = GL_LIGHT3;
+	        break;
 	    case (4) :
-	        return(GL_LIGHT4);
-	        // break;
+	        retLight = GL_LIGHT4;
+	        break;
 	    case (5) :
-	        return(GL_LIGHT5);
-	        // break;
+	        retLight = GL_LIGHT5;
+	        break;
 	    case (6) :
-	        return(GL_LIGHT6);
-	        // break;
+	        retLight = GL_LIGHT6;
+	        break;
 	    case (7) :
-	        return(GL_LIGHT7);
-	        // break;
+	        retLight = GL_LIGHT7;
+	        break;
 	    default :
 	        error("GEM: Unable to allocate world_light");
 	        return((GLenum)0);
 	        // break;
     }
-//    return(0);
+    return(retLight);
 }
 
 /////////////////////////////////////////////////////////
@@ -701,36 +793,44 @@ GLenum GemMan :: requestLight()
 /////////////////////////////////////////////////////////
 void GemMan :: freeLight(GLenum lightNum)
 {
+	int i = 0;
     switch(lightNum)
     {
     	case(GL_LIGHT0):
-    	    s_lights[0] = 0;
-    	    return;
+    	    i = 0;
+			break;
     	case(GL_LIGHT1):
-    	    s_lights[1] = 0;
-    	    return;
+			i = 1;
+			break;
     	case(GL_LIGHT2):
-    	    s_lights[2] = 0;
-    	    return;
+			i = 2;
+			break;
     	case(GL_LIGHT3):
-    	    s_lights[3] = 0;
-    	    return;
+			i = 3;
+			break;
     	case(GL_LIGHT4):
-    	    s_lights[4] = 0;
-    	    return;
+			i = 4;
+			break;
     	case(GL_LIGHT5):
-    	    s_lights[5] = 0;
-    	    return;
+			i = 5;
+			break;
     	case(GL_LIGHT6):
-    	    s_lights[6] = 0;
-    	    return;
+			i = 6;
+			break;	
     	case(GL_LIGHT7):
-    	    s_lights[7] = 0;
-    	    return;
+			i = 7;
+			break;
     	default:
     	    error("GEM: Error freeing a light - bad number");
-    	    break;
+			return;
+			// break;
     }
+	s_lights[i]--;
+	if (s_lights[i] < 0)
+	{
+		error("GEM: light ref count below zero: %d", i);
+		s_lights[i] = 0;
+	}
 }
 
 /////////////////////////////////////////////////////////
