@@ -6,7 +6,7 @@ LOG
        - code to load in and resize an image
        - part of GEM
 
-    Copyright (c) 1997-1998 Mark Danks. mdanks@cybermed.ucsd.edu
+    Copyright (c) 1997-1999 Mark Danks. mark@danks.org
     For information on usage and redistribution, and for a DISCLAIMER OF ALL
     WARRANTIES, see the file, "GEM.LICENSE.TERMS" in this distribution.
 
@@ -15,7 +15,9 @@ LOG
 #ifndef INCLUDE_GEMPIXIMAGELOAD_H_
 #define INCLUDE_GEMPIXIMAGELOAD_H_
 
-#include "Base/GemPixUtil.h"
+#include "Base/GemExportDef.h"
+
+struct imageStruct;
 
 // image2mem() reads an image file into memory
 //   and a pointer to an imageStruct
@@ -26,6 +28,8 @@ LOG
 // 
 //   automatically allocates the memory for the user
 //
+// This can read TIFF, SGI, and JPG images
+//
 GEM_EXTERN extern imageStruct *image2mem(const char *filename);
 
 #endif
