@@ -2,11 +2,11 @@
 //
 // GEM - Graphics Environment for Multimedia
 //
-// mdanks@cybermed.ucsd.edu
+// mark@danks.org
 //
 // Implementation file
 //
-//    Copyright (c) 1997-1998 Mark Danks.
+//    Copyright (c) 1997-1999 Mark Danks.
 //    For information on usage and redistribution, and for a DISCLAIMER OF ALL
 //    WARRANTIES, see the file, "GEM.LICENSE.TERMS" in this distribution.
 //
@@ -14,8 +14,6 @@
 
 #include "GemState.h"
 
-#include <stdio.h>
-
 /////////////////////////////////////////////////////////
 //
 // GemState
@@ -25,13 +23,6 @@
 //
 /////////////////////////////////////////////////////////
 GemState :: GemState()
-    	  : dirty(0), inDisplayList(0), lighting(0), smooth(0), texture(0),
-    	    image(0), texCoords(NULL), numTexCoords(0)
-{ }
-
-/////////////////////////////////////////////////////////
-// Destructor
-//
-/////////////////////////////////////////////////////////
-GemState :: ~GemState()
+		  : dirty(0), inDisplayList(0), lighting(0), smooth(0), texture(0),
+    		image(0), texCoords(0), numTexCoords(0)
 { }
