@@ -4,7 +4,7 @@ LOG
 
     A light
 
-    Copyright (c) 1997-1998 Mark Danks. mark@danks.org
+    Copyright (c) 1997-1999 Mark Danks. mark@danks.org
     For information on usage and redistribution, and for a DISCLAIMER OF ALL
     WARRANTIES, see the file, "GEM.LICENSE.TERMS" in this distribution.
 
@@ -37,23 +37,13 @@ class GEM_EXTERN light : public world_light
 
 	    //////////
 	    // Constructor
-    	light();
+    	light(t_floatarg lightNum);
     	
     protected:
     	
     	//////////
     	// Destructor
     	virtual ~light();
-
-   	    //////////
-    	// Set the light's position
-    	void	    	lightPositionMess(float x, float y, float z);
-
-    private:
-    	    
-    	//////////
-    	// Static member functions
-    	static void 	lightPositionMessCallback(void *data, t_floatarg x, t_floatarg y, t_floatarg z);
 };
 
 #endif	// for header file
